/**
 *
 * Magnolia Criteria API (http://www.openmindlab.com/lab/products/mgnlcriteria.html)
 * Copyright (C)2009 - 2010, Openmind S.r.l. http://www.openmindonline.it
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.sourceforge.openutils.mgnlcriteria.jcr.query.criterion;

import java.util.Calendar;

import net.sourceforge.openutils.mgnlcriteria.jcr.query.Criteria;
import net.sourceforge.openutils.mgnlcriteria.jcr.query.JCRQueryException;


/**
 * superclass for "simple" comparisons (with XPATH binary operators)
 * @author Federico Grilli
 * @version $Id: SimpleExpression.java 2050 2010-02-24 15:35:30Z fgiust $
 */
public class SimpleExpression implements Criterion
{

    private static final long serialVersionUID = -1104419394978535803L;

    private final String propertyName;

    private final Object value;

    private final String op;

    protected SimpleExpression(String propertyName, Object value, String op)
    {
        this.propertyName = propertyName;
        this.value = value;
        this.op = op;
    }

    @Override
    public String toString()
    {
        return propertyName + getOp() + value;
    }

    protected final String getOp()
    {
        return op;
    }

    public String toXPathString(Criteria criteria) throws JCRQueryException
    {
        StringBuilder fragment = new StringBuilder();
        fragment.append(" (");

        if (value instanceof String)
        {
            fragment.append(propertyName).append(getOp());
            fragment.append("'" + value + "') ");
        }
        else if (value instanceof Number)
        {
            fragment.append(propertyName).append(getOp());
            fragment.append(value + ") ");
        }
        else if (value instanceof Character)
        {
            fragment.append(propertyName).append(getOp());
            fragment.append("'" + Character.toString((Character) value) + "') ");
        }
        else if (value instanceof Boolean)
        {
            if ((Boolean) value)
            {
                fragment.append(propertyName).append(getOp());
                fragment.append(value + ") ");
            }
            else
            {
                // false should also match a missing boolean property
                fragment.append("(");
                fragment.append(propertyName).append(getOp());

                fragment.append(value + ") or not(").append(propertyName + " ))");
            }
        }
        else if (value instanceof Calendar)
        {
            fragment.append(propertyName).append(getOp());
            Calendar cal = (Calendar) value;
            if ("=".equals(op))
            {
                // trick to make it work with dates
                return Restrictions.between(propertyName, value, value).toXPathString(null);

            }
            else
            {
                StringBuilder date = new StringBuilder();
                date.append(cal.get(Calendar.YEAR)).append(HYPHEN).append(
                    cal.get(Calendar.MONTH) < MONTH_MAX
                        ? "0" + (cal.get(Calendar.MONTH) + 1)
                        : cal.get(Calendar.MONTH) + 1).append(HYPHEN).append(
                    cal.get(Calendar.DAY_OF_MONTH) < DAY_MAX ? "0" + cal.get(Calendar.DAY_OF_MONTH) : cal
                        .get(Calendar.DAY_OF_MONTH)).append(MIDNIGHT);
                fragment.append(XS_DATETIME_FUNCTION + "('" + date.toString() + "')) ");
            }
        }
        else if (value != null)
        {
            fragment.append(propertyName).append(getOp());

            // just use the toString() of the given object
            fragment.append("'" + value + "') ");
        }
        log.debug("xpathString is {} ", fragment);
        return fragment.toString();
    }
}
